﻿using DamageAssesment.Api.Responses.Interfaces;
using DamageAssesment.Api.Responses.Models;
using Microsoft.Extensions.Primitives;
using Newtonsoft.Json;


namespace DamageAssesment.Api.Responses.Services
{
    public class AnswerServiceProvider : ServiceProviderBase, IAnswerServiceProvider
    {
        public AnswerServiceProvider(IConfiguration configuration, IHttpUtil httpUtil, ILogger<AnswerServiceProvider> logger) : base(configuration, httpUtil, logger, configuration.GetValue<string>("RessourceSettings:Answer"), configuration.GetValue<string>("EndPointSettings:AnswerUrlBase"))
        {
        }
        public async Task<List<Answer>> getAnswersAsync(string token)
        {
            try
            {
                var responseJsonString = await httpUtil.SendAsync(HttpMethod.Get, url, null, token);
                var answers = JsonConvert.DeserializeObject<List<Answer>>(responseJsonString);

                if (answers == null || !answers.Any())
                    return new List<Answer>();
                else return answers;
            }
            catch (Exception ex)
            {
                logger?.LogError($"Exception Found : {ex.Message} - Ref: AnswerServiceProvider.getAnswersAsync()");
                return new List<Answer>();
            }
        }

        public async Task<List<Answer>> GetAnswersByResponseIdAsync(int responseId, string token)
        {
            try
            {
                url = urlBase + string.Format(configuration.GetValue<string>("RessourceSettings:AnswerByResponse"), responseId);
                var responseJsonString = await httpUtil.SendAsync(HttpMethod.Get, url, null,token);
                var answers = JsonConvert.DeserializeObject<List<Answer>>(responseJsonString);

                if (answers == null || !answers.Any())
                    return new List<Answer>();
                else return answers;
            }
            catch (Exception ex)
            {
                logger?.LogError($"Exception Found : {ex.Message} - Ref: AnswerServiceProvider.GetAnswersByResponseId()");
                return new List<Answer>();
            }
        }

        public async Task<Answer> PostAnswersAsync(Answer answer, string token )
        {
            try
            {
                var requestJsonString = JsonConvert.SerializeObject(answer);
                var responseJsonString = await httpUtil.SendAsync(HttpMethod.Post, url, requestJsonString, token);
                var answers = JsonConvert.DeserializeObject<Answer>(responseJsonString);

                if (answers == null)
                {
                    logger?.LogError($"Answers cannot be added - Ref: AnswerServiceProvider.PostAnswersAsync()");
                    return null;
                }
                else return answers;
            }
            catch (Exception ex)
            {
                logger?.LogError($"Exception Found : {ex.Message} - Ref: AnswerServiceProvider.PostAnswersAsync()");
                return null;
            }
        }
    }
}
